# YouTube
YT = "https://www.youtube.com/"
YT_VID_URL = YT + "watch?v="
YT_CHANNEL_URL = YT + "channel/"

# Telega.ph
TELEGRA_PH = "https://telegra.ph"
TELEGRA_PH_HEADERS = {
    "authority": "telegra.ph",
    "accept": "application/json, text/javascript, */*; q=0.01",
    "x-requested-with": "XMLHttpRequest",
    "user-agent": "Python_telegraph_poster/0.1",
    "origin": TELEGRA_PH,
    "referer": f"{TELEGRA_PH}/",
    "accept-language": "en-US,en;q=0.9",
}
