__all__ = ["audio", "video", "photo"]

from typing import Tuple


audio: Tuple[str] = (".mp3", ".flac", ".wav", ".m4a")
video: Tuple[str] = (".mkv", ".mp4", ".webm")
photo: Tuple[str] = (".jpg", ".jpeg", ".png", ".bmp", ".webp")
